Package: topr
Title: Create Custom Plots for Viewing Genetic Association Results
Version: 2.0.4
Authors@R: c(
    person("Thorhildur", "Juliusdottir", role = c("cph","aut", "cre"),email = "totajuliusd@gmail.com"),
    person("Andri", "Stefansson", role = "aut", email = "andrimarstefansson@gmail.com"),
    person("Kyle","Scott", role="ctb", email="kms309@miami.edu")
    )
URL: https://github.com/totajuliusd/topr
BugReports: https://github.com/totajuliusd/topr/issues
Description: A collection of functions for visualizing,exploring and annotating genetic association results.Association results from multiple traits can be viewed simultaneously along with gene annotation, over the entire genome (Manhattan plot) or in the more detailed regional view.
License: LGPL (>= 3)
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
RoxygenNote: 7.3.3
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, markdown
Config/testthat/edition: 3
Imports: ggplot2 (>= 3.3.2), dplyr (>= 1.0.2), stringr (>= 1.4.0),
        scales (>= 1.1.1), ggrepel, egg, gridExtra, magrittr (>= 1.5),
        utils, tidyr, enshuman, purrr, tibble
Depends: R (>= 4.1.0)
NeedsCompilation: no
Packaged: 2026-04-13 09:30:44 UTC; tjuliusdottir
Author: Thorhildur Juliusdottir [cph, aut, cre],
  Andri Stefansson [aut],
  Kyle Scott [ctb]
Maintainer: Thorhildur Juliusdottir <totajuliusd@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-13 10:10:02 UTC
Built: R 4.4.3; ; 2026-04-13 11:39:17 UTC; unix
