# Generated by roxygen2: do not edit by hand

export(annotate_with_nearest_gene)
export(create_snpset)
export(create_snpset_code)
export(effect_plot)
export(effectplot)
export(flip_to_positive_allele_for_dat1)
export(foresttopr)
export(get_best_snp_per_MB)
export(get_gene)
export(get_gene_coords)
export(get_genes_by_Gene_Symbol)
export(get_genes_in_region)
export(get_lead_snps)
export(get_overlapping_snps_by_pos)
export(get_sign_and_sugg_loci)
export(get_snps_within_region)
export(get_snpset)
export(get_snpset_code)
export(get_top_snp)
export(get_topr_colors)
export(get_topr_old_colors)
export(locuszoom)
export(manhattan)
export(manhattanExtra)
export(match_alleles)
export(match_by_alleles)
export(match_by_pos)
export(qqtopr)
export(regionplot)
import(dplyr)
import(ggplot2)
import(utils)
importFrom(grDevices,col2rgb)
importFrom(magrittr,"%>%")
importFrom(stats,setNames)
importFrom(utils,globalVariables)
